// Export.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "..\FileStruct.h"

#define ATLASSERT(x)
#include "atlwfile.h"
using namespace ATL;

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved )
{
   switch( ul_reason_for_call ) {
   case DLL_PROCESS_ATTACH:
   case DLL_THREAD_ATTACH:
   case DLL_THREAD_DETACH:
   case DLL_PROCESS_DETACH:
      ::DisableThreadLibraryCalls((HINSTANCE)hModule);
      break;
   }
   return TRUE;
}

//----------------------

extern "C" {
   VOID APIENTRY Mess_GetTitle(LPWSTR szCaption, SHORT nChars);
   BOOL APIENTRY Mess_Execute(tExtensionArgs *args);
};
void SaveXML(LPCTSTR szFilename, tExtensionArgs *args);

//----------------------

VOID APIENTRY Mess_GetTitle(LPWSTR szCaption, SHORT nChars)
{
   wcscpy(szCaption, L"Export");	
}

BOOL APIENTRY Mess_Execute(tExtensionArgs *args)
{
   ATLASSERT(args);
   if( args->version<VERSION ) return FALSE;
   if( args->size<sizeof(tExtensionArgs) ) return FALSE;

   LPCTSTR szFilter = 
	   _T("XML Files (*.xml)\0*.xml\0")
	   _T("");
   TCHAR szFileName[MAX_PATH];
   TCHAR szFileTitle[MAX_PATH];
   szFileName[0] = '\0';
   szFileTitle[0] = '\0';

   OPENFILENAME m_ofn = { 0 };
   m_ofn.lStructSize = sizeof(m_ofn);
   m_ofn.Flags = OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_ENABLESIZING;
   m_ofn.hInstance = ::GetModuleHandle(NULL);
   m_ofn.hwndOwner = args->hWnd;
   m_ofn.lpstrTitle = _T("Export As...");
   m_ofn.lpstrFilter = szFilter;
   m_ofn.nFilterIndex = 1;
   m_ofn.lpstrFile = szFileName;
   m_ofn.nMaxFile = MAX_PATH;
   m_ofn.lpstrFileTitle = szFileTitle;
   m_ofn.nMaxFileTitle = MAX_PATH;
   m_ofn.lpstrDefExt = _T("xml");

   BOOL bRet;
   bRet = ::GetSaveFileName(&m_ofn);
   if(bRet) {
      SaveXML(szFileName,args);
   };
   return TRUE;
}

// ------------------------------ XML -------

#define MAX_STRING 400

LPCTSTR Convert2XML(LPCTSTR str)
{
   static TCHAR szBuffer[MAX_STRING+50];

   *szBuffer = _T('\0');
   if( str==NULL ) return szBuffer;
   int len = _tcslen(str);
   if( len>300 ) return szBuffer;

   LPTSTR pSrc = (LPTSTR)str;
   LPTSTR pDst = (LPTSTR)szBuffer;
   for( int i=0; i<len; i++ ) {
      TCHAR ch = *pSrc;
      pSrc = ::CharNext(pSrc);
      switch( ch ) {
      case _T('<'):
         *pDst++ = _T('&');
         *pDst++ = _T('l');
         *pDst++ = _T('t');
         *pDst++ = _T(';');
         break;
      case _T('>'):
         *pDst++ = _T('&');
         *pDst++ = _T('g');
         *pDst++ = _T('t');
         *pDst++ = _T(';');
         break;
      case _T('&'):
         *pDst++ = _T('&');
         *pDst++ = _T('a');
         *pDst++ = _T('m');
         *pDst++ = _T('p');
         *pDst++ = _T(';');
         break;
      case _T('\"'):
         *pDst++ = _T('&');
         *pDst++ = _T('q');
         *pDst++ = _T('u');
         *pDst++ = _T('o');
         *pDst++ = _T('t');
         *pDst++ = _T(';');
         break;
      case _T('\''):
         *pDst++ = _T('&');
         *pDst++ = _T('a');
         *pDst++ = _T('p');
         *pDst++ = _T('o');
         *pDst++ = _T('s');
         *pDst++ = _T(';');
         break;
      default:
         if( ch>0 && ch<128 ) {
            *pDst++ = ch;
            // Oops, not very good to MBCS...
         }
         else {
            TCHAR szBuffer[10];
            ::wsprintf(szBuffer, _T("&#%d;"), (unsigned char)ch);
            LPCTSTR pSrc = szBuffer;
            while( *pSrc ) *pDst++ = *pSrc++;
         }
      };
   };
   *pDst = _T('\0');
   return szBuffer;
};

LPCTSTR Convert2XML(LPCWSTR str)
{
   TCHAR szBuffer[MAX_STRING];
   ::wsprintf(szBuffer, _T("%ls"), str);
   return Convert2XML(szBuffer);
}

inline void WriteString(CFile &f, LPCTSTR str)
{
   f.Write(str, _tcslen(str));
};

void SaveXML(LPCTSTR szFilename, tExtensionArgs *args)
{
   if( args==NULL ) return;
   TCHAR szBuffer[MAX_STRING];
   CFile f;
   int i;
   if( f.Create(szFilename)==FALSE ) return;
   //
   WriteString(f,_T("<?xml version=\"1.0\"?>\n"));
   WriteString(f,_T("<FileMess>\n"));
   // Store the filters
   WriteString(f,_T("<Filters>\n"));
   for( i=0; i<args->nNumFilters; i++ ) {
      tFilter &filter = args->Filters[i];
      ::wsprintf(szBuffer, _T("  <Filter id=\"%d\">\n"), i+1);
      WriteString(f,szBuffer);
      //
      ::wsprintf(szBuffer, _T("    <Title>%s</Title>\n"), Convert2XML(filter.szTitle));
      WriteString(f,szBuffer);
      ::wsprintf(szBuffer, _T("    <Author>%s</Author>\n"), Convert2XML(filter.szAuthor));
      WriteString(f,szBuffer);
      ::wsprintf(szBuffer, _T("    <Album>%s</Album>\n"), Convert2XML(filter.szAlbum));
      WriteString(f,szBuffer);
      ::wsprintf(szBuffer, _T("    <SourcePath>%s</SourcePath>\n"), Convert2XML(filter.szSourcePath));
      WriteString(f,szBuffer);
      ::wsprintf(szBuffer, _T("    <TargetPath>%s</TargetPath>\n"), Convert2XML(filter.szTargetPath));
      WriteString(f,szBuffer);
      ::wsprintf(szBuffer, _T("    <Pattern>%s</Pattern>\n"), Convert2XML(filter.szPattern));
      WriteString(f,szBuffer);
      //
      WriteString(f,_T("  </Filter>\n"));
   };
   WriteString(f,_T("</Filters>\n"));
   // Store all the macros
   WriteString(f,_T("<Macros>\n"));
   for( i=0; i<args->nNumMacros; i++ ) {
      tMacro &macro = args->Macros[i];
      ::wsprintf(szBuffer, _T("  <Macro id=\"%d\">\n"), i+1);
      WriteString(f,szBuffer);
      //
      ::wsprintf(szBuffer, _T("    <Token>%s</Token>\n"), Convert2XML(macro.szMacro));
      WriteString(f,szBuffer);
      ::wsprintf(szBuffer, _T("    <Replacement>%s</Replacement>\n"), Convert2XML(macro.szReplacement));
      WriteString(f,szBuffer);
      //
      WriteString(f,_T("  </Macro>\n"));
   };
   WriteString(f,_T("</Macros>\n"));
   //
   WriteString(f,_T("</FileMess>\n"));
   // Done
   f.Close();
};
