#if !defined(AFX_EXTENSIONS_H__20000421_3637_EEDB_002B_0080AD509054__INCLUDED_)
#define AFX_EXTENSIONS_H__20000421_3637_EEDB_002B_0080AD509054__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef VOID (CALLBACK* LPFNGETCAPTION)(LPWSTR,SHORT);
typedef BOOL (CALLBACK* LPFNCALL)(tExtensionArgs*);

// The CExtension class encapsulates the add-in extension
// for the application.
class CExtension
{
protected:
   CString m_strFilename;
   HINSTANCE m_hInstance;
   UINT m_ID;
   //
   LPFNCALL m_lpfnProjectInit;
   LPFNCALL m_lpfnExecute;
   LPFNCALL m_lpfnRefresh;
   LPFNCALL m_lpfnAcceptFile;
   LPFNCALL m_lpfnFileOperation;
   LPFNCALL m_lpfnSave;

public:
   CExtension()
   {
      m_hInstance = NULL;
      m_lpfnProjectInit =
      m_lpfnExecute =
      m_lpfnRefresh =
      m_lpfnAcceptFile = 
      m_lpfnFileOperation =
      m_lpfnSave = NULL;
      m_ID = -1;
   };
   
   ~CExtension()
   { 
      // NOTE: Do not FreeLibrary the extension. System will do this, and
      //       it will screw up our reference to m_hInstance because STL
      //       clones this class around (destroys/creates copies).
   };

   UINT GetID() const { return m_ID; };
   void SetID(UINT ID) { m_ID = ID; };

   boolean Load(LPCTSTR szFilename)
   {
      ATLASSERT(szFilename);
      m_strFilename = szFilename;
      if( m_strFilename.IsEmpty() ) return false;

      m_hInstance = ::LoadLibrary(szFilename);      
      if( m_hInstance<(HINSTANCE)32 ) return false;

      m_lpfnProjectInit = (LPFNCALL)::GetProcAddress(m_hInstance, _T("Mess_ProjectInit"));
      m_lpfnExecute = (LPFNCALL)::GetProcAddress(m_hInstance, _T("Mess_Execute"));
      m_lpfnAcceptFile = (LPFNCALL)::GetProcAddress(m_hInstance, _T("Mess_AcceptFile"));
      m_lpfnFileOperation = (LPFNCALL)::GetProcAddress(m_hInstance, _T("Mess_FileOperation"));
      m_lpfnRefresh = (LPFNCALL)::GetProcAddress(m_hInstance, _T("Mess_Refresh"));
      m_lpfnSave = (LPFNCALL)::GetProcAddress(m_hInstance, _T("Mess_Save"));

      return true;
   };

   //----------------------

   CString GetTitle()
   {
      ATLASSERT(m_hInstance);
      USES_CONVERSION;
      if( m_hInstance==NULL ) return CString();
      
      LPFNGETCAPTION lpfnGetCaption = (LPFNGETCAPTION)::GetProcAddress(m_hInstance, _T("Mess_GetTitle"));
      if( lpfnGetCaption==NULL ) return _T("");
      
      WCHAR szTitle[64];
      lpfnGetCaption(szTitle,64);
      return CString(W2CT(szTitle));
   };

   //----------------------

   BOOL ProjectInit(HWND hWnd, CMessHandler &mess)
   {
      ATLASSERT(m_hInstance);
      if( m_hInstance==NULL ) return FALSE;    
      if( m_lpfnProjectInit==NULL ) return FALSE;
      
      tExtensionArgs args;
      CreateArguments(args, hWnd, mess);
      return m_lpfnProjectInit(&args);
   };

   BOOL Execute(HWND hWnd, CMessHandler &mess, UINT SelectedID)
   {
      ATLASSERT(m_hInstance);
      if( m_hInstance==NULL ) return FALSE;    
      if( m_lpfnExecute==NULL ) return FALSE;
      
      tExtensionArgs args;
      CreateArguments(args, hWnd, mess);
      args.GroupID = SelectedID;
      return m_lpfnExecute(&args);
   };

   BOOL AcceptFile(HWND hWnd, CMessHandler &mess, LPCTSTR szFileName)
   {
      USES_CONVERSION;
      ATLASSERT(m_hInstance);
      if( m_hInstance==NULL ) return FALSE;    
      if( m_lpfnAcceptFile==NULL ) return FALSE;
      
      tExtensionArgs args;
      CreateArguments(args, hWnd, mess);
      wcscpy( args.szFileName, T2CW(szFileName) );
      return m_lpfnAcceptFile(&args);
   };

   BOOL FileOperation(HWND hWnd, CMessHandler &mess, FILEACTION action, UINT GroupID, LPCTSTR szFileName)
   {
      USES_CONVERSION;
      ATLASSERT(m_hInstance);
      if( m_hInstance==NULL ) return FALSE;    
      if( m_lpfnFileOperation==NULL ) return FALSE;
      
      tExtensionArgs args;
      CreateArguments(args, hWnd, mess);
      args.GroupID = GroupID;
      args.action = action;
      wcscpy( args.szFileName, T2CW(szFileName) );
      return m_lpfnFileOperation(&args);
   };

   BOOL Refresh(HWND hWnd, CMessHandler &mess)
   {
      ATLASSERT(m_hInstance);
      if( m_hInstance==NULL ) return FALSE;    
      if( m_lpfnRefresh==NULL ) return FALSE;
      
      tExtensionArgs args;
      CreateArguments(args, hWnd, mess);
      return m_lpfnRefresh(&args);
   };

   BOOL Save(HWND hWnd, CMessHandler &mess)
   {
      ATLASSERT(m_hInstance);
      if( m_hInstance==NULL ) return FALSE;    
      if( m_lpfnSave==NULL ) return FALSE;
      
      tExtensionArgs args;
      CreateArguments(args, hWnd, mess);
      return m_lpfnSave(&args);
   };

   //----------------------

private:
   void CreateArguments(tExtensionArgs &args, HWND hWnd, CMessHandler &mess)
   {
      ATLASSERT(::IsWindow(hWnd));
      USES_CONVERSION;
      ::ZeroMemory(&args,sizeof(args));
      args.version = VERSION;
      args.size = sizeof(args);
      args.hWnd = hWnd;
      wcscpy( args.szProjectFile, T2CW(mess.FileName()) );
      args.nNumMacros = mess.m_Macros.size();
      args.Macros = args.nNumMacros==0 ? NULL : &mess.m_Macros[0];
      args.nNumFilters = mess.m_Filters.size();
      args.Filters = args.nNumFilters==0 ? NULL : &mess.m_Filters[0];
   };
};


#endif // !defined(AFX_EXTENSIONS_H__20000421_3637_EEDB_002B_0080AD509054__INCLUDED_)

