#if !defined(AFX_FILESTRUCT_H__20000421_619E_818E_A95B_0080AD509054__INCLUDED_)
#define AFX_FILESTRUCT_H__20000421_619E_818E_A95B_0080AD509054__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

///////////////////////////////////////////////////////
//
// The FileMess structure used for extensions.
//
// Author: Bjarke Viksoe (bjarke@viksoe.dk)
//
// Use these definitions to write custom add-ins to
// the File Mess application.
// Add-ins are automatically added to the application's
// Tools menu.
//
// THIS IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
// BJARKE VIKSOE DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED,
// INCLUDING THE WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE. IN NO EVENT SHALL BJARKE VIKSOE 
// BE LIABLE FOR ANY DAMAGES WHATSOEVER INCLUDING
// DIRECT, INDIRECT, INCIDENTAL, CONSEQUENTIAL, LOSS OF BUSINESS
// PROFITS OR SPECIAL DAMAGES, EVEN IF BJARKE VIKSOE
// HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
///////////////////////////////////////////////////////


#define VERSION 1


///////////////////////////
// Macro defintion

#define MAX_MACRO 300

typedef struct tagMacro
{
   WCHAR szMacro[MAX_MACRO];
   WCHAR szReplacement[MAX_MACRO];
} tMacro;


///////////////////////////
// File Filter structure

#define MAX_FILEDATA 200
#define MAX_FILEPATTERN 40

typedef enum FILEACTION 
{
   FILEMOVE=1, 
   FILECOPY=2, 
   FILEDELETE=4
};

typedef struct tagFilter
{
   DWORD  ID;
   USHORT Version;
   //
   FILEACTION action;
   SYSTEMTIME DateCreated;
   SYSTEMTIME DateModified;
   //
   WCHAR szSourcePath[MAX_PATH];
   WCHAR szTargetPath[MAX_PATH];
   WCHAR szPattern[MAX_FILEPATTERN];
   //
   WCHAR  szTitle[MAX_FILEDATA];  // the author of the 
   WCHAR  szAuthor[MAX_FILEDATA]; // who wrote the article
   WCHAR  szAlbum[MAX_FILEDATA];  // name of the album
   SYSTEMTIME Date;          // date it was produced
   long   nFromPicture;  // both 0 if not defined, otherwise this
   long   nToPicture;    // scans the filename for a numbered pattern
   //
   long  nFiles;          // number of file matches
   long  nOrigFiles;      // for internal book-keeping
   DWORD dwTotalFileSize; // total number of bytes used by all files
   long  nNewFiles;       // number of new files
   long  nMissingFiles;   // number of missing files if numbered pattern (# pattern)
} tFilter;


/////////////////////////
// Application options

typedef struct tagOptions
{
   WCHAR szExtensions[128];
   BOOL bRemoveUnmatchedFiles; // remove files from folder if no match
   BOOL bListenForChanges;     // setup folder notification
   BOOL bIncludeSubFolders;    // recurse
   BOOL bIgnoreDuplicates;     // don't copy file if already exists
   BOOL bUseRecycleBin;        // delete files to recycle bin
   BOOL bCheckDiskSpace;       // warn if we run out of disk space
} tOptions;


////////////////////////////////////
// Application extension structure

typedef struct tagExtensionArgs
{
   USHORT version;
   DWORD size;
   //
   HWND hWnd;
   WCHAR szProjectFile[MAX_PATH];
   //
   UINT GroupID;
   FILEACTION action;
   WCHAR szFileName[MAX_PATH];
   //
   tOptions *Options;
   //
   tFilter *Filters;
   SHORT nNumFilters;
   //
   tMacro *Macros;
   SHORT nNumMacros;
} tExtensionArgs;

///
// To create an extension of the FileMess application
// define the following DLL:
//
// #include <windows.h>
// #include "FileStruct.h"
//
// VOID APIENTRY Mess_GetTitle(LPWSTR szName, SHORT nMaxChars)
// {
//    wcscpy( szName, L"Test Extension" );
// }
//
// BOOL APIENTRY Mess_Execute(tExtensionArgs *arg)
// {
//    ::MessageBox(NULL, "Activated", "Extension", MB_ICONINFORMATION );
//    return TRUE;
// }
//
// Give it a ".ext" filename extension and put it in the
// FileMess application folder.
// Other available callback functions are:
//
// BOOL APIENTRY Mess_ProjectInit(tExtensionArgs *arg)
// BOOL APIENTRY Mess_Refresh(tExtensionArgs *arg)
// BOOL APIENTRY Mess_Save(tExtensionArgs *arg)
//
///

#endif // !defined(AFX_FILESTRUCT_H__20000421_619E_818E_A95B_0080AD509054__INCLUDED_)

