// macrosdlg.h : interface of the CMacrosDlg class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MACROSDLG_H__5FF6DF87_D992_4D6C_B87F_733856B87CE0__INCLUDED_)
#define AFX_MACROSDLG_H__5FF6DF87_D992_4D6C_B87F_733856B87CE0__INCLUDED_

#include "FileStruct.h"
#include "MessHandler.h"

class CMacrosDlg : public CDialogImpl<CMacrosDlg>
{
public:
   enum { IDD = IDD_MACROS };

   CEdit m_ctlMacro;
   CEdit m_ctlReplacement;
   CListViewCtrl m_ctlList;
   CMessHandler *m_mess;

   BEGIN_MSG_MAP(CMacrosDlg)
      MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
      COMMAND_ID_HANDLER(IDOK, OnCloseCmd)
      COMMAND_ID_HANDLER(IDCANCEL, OnCloseCmd)
      COMMAND_ID_HANDLER(IDC_BROWSE, OnBrowse)
      COMMAND_ID_HANDLER(IDC_ADD, OnAdd)
      COMMAND_ID_HANDLER(IDC_UPDATE, OnUpdate)
      COMMAND_ID_HANDLER(IDC_DELETE, OnDelete)
      NOTIFY_HANDLER(IDC_MACROS, LVN_ITEMCHANGED, OnListSelect)
   END_MSG_MAP()

   LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      ATLASSERT(m_mess);

      m_ctlMacro.Attach(GetDlgItem(IDC_MACRO));
      m_ctlMacro.LimitText(MAX_MACRO-2);
      m_ctlReplacement.Attach(GetDlgItem(IDC_REPLACEMENT));
      m_ctlReplacement.LimitText(MAX_MACRO);
      m_ctlList.Attach(GetDlgItem(IDC_MACROS));

      CRect rc;
      m_ctlList.GetClientRect(&rc);
      int nWidth = (rc.Width()/2) - 6;
      m_ctlList.InsertColumn(0, _T("1"), LVCFMT_LEFT, nWidth, 1);
      m_ctlList.InsertColumn(1, _T("2"), LVCFMT_LEFT, nWidth, 1);
      m_ctlList.SetExtendedListViewStyle(LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);

      PopulateList();
      m_ctlList.SelectItem(0);

      CenterWindow(GetParent());
      return TRUE;
   }

   LRESULT OnCloseCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      EndDialog(wID);
      return 0;
   }

   LRESULT OnBrowse(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CFolderDialog dlg(m_hWnd);
      if( dlg.DoModal()==IDOK ) {
         m_ctlReplacement.SetWindowText(dlg.GetFolderPath());
      }
      m_ctlReplacement.SetFocus();
      return 0;
   }

   LRESULT OnAdd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      ATLASSERT(m_mess);
      ATLASSERT(::IsWindow(m_ctlList));
      USES_CONVERSION;

      TCHAR szMacro[MAX_MACRO];
      TCHAR szReplacement[MAX_MACRO];
      m_ctlMacro.GetWindowText(szMacro, MAX_MACRO);
      m_ctlReplacement.GetWindowText(szReplacement, MAX_MACRO);

      CString sMacro(szMacro);
      CString sReplacement(szReplacement);
      sMacro.TrimLeft();
      sMacro.TrimRight();

      if( sMacro.GetLength()==0 ) {
         CString err;
         err.LoadString(IDS_ERR_MACRONAME);
         MessageBox(err,NULL, MB_OK | MB_ICONWARNING);
         return 0;
      };

      // Make sure to add % characters around macro name
      if( sMacro.GetAt(0)!=_T('%') ) sMacro = _T('%') + sMacro;
      if( sMacro.GetAt(sMacro.GetLength()-1)!=_T('%') ) sMacro += _T('%');
      
      // Check for duplicate
      for( int i=0; i<m_mess->m_Macros.size(); i++ ) {
         if( _tcsicmp(sMacro, W2CT(m_mess->m_Macros[i].szMacro) )==0 ) {
            CString err;
            err.LoadString(IDS_ERR_DUPLICATEMACRO);
            MessageBox(err,NULL, MB_OK | MB_ICONWARNING);
            return 0;
         };
      };

      tMacro macro = { 0 };
      wcscpy( macro.szMacro, T2CW(sMacro) );
      wcscpy( macro.szReplacement, T2CW(sReplacement) );
      m_mess->m_Macros.push_back(macro);
      m_mess->SetModified();
      PopulateList();

      // Select new macro, and make sure we update it if name was changed
      m_ctlList.SelectItem(m_ctlList.GetItemCount()-1);

      return 0;
   }

   LRESULT OnUpdate(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      USES_CONVERSION;
      int idx = m_ctlList.GetSelectedIndex();
      if( idx<0 ) return 0;
      TCHAR szMacro[MAX_MACRO];
      TCHAR szReplacement[MAX_MACRO];
      m_ctlMacro.GetWindowText(szMacro, MAX_MACRO);
      m_ctlReplacement.GetWindowText(szReplacement, MAX_MACRO);

      tMacro macro;
      wcscpy( macro.szMacro, T2CW(szMacro) );
      wcscpy( macro.szReplacement, T2CW(szReplacement) );

      m_mess->m_Macros[idx] = macro;
      m_mess->SetModified();
      PopulateList();
      m_ctlList.SelectItem(idx);

      return 0;
   }

   LRESULT OnDelete(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      ATLASSERT(m_mess);
      ATLASSERT(::IsWindow(m_ctlList));
      int idx = m_ctlList.GetSelectedIndex();
      if( idx<0 ) return 0;
      m_mess->m_Macros.erase(m_mess->m_Macros.begin()+idx);
      m_mess->SetModified();
      PopulateList();
      m_ctlList.SelectItem(0);
      return 0;
   }

   LRESULT OnListSelect(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
   {
      ATLASSERT(m_mess);
      ATLASSERT(::IsWindow(m_ctlList));
     
      int idx = m_ctlList.GetSelectedIndex();
      if( idx<0 ) {
         m_ctlMacro.SetWindowText(_T(""));
         m_ctlReplacement.SetWindowText(_T(""));
         ::EnableWindow( GetDlgItem(IDC_DELETE), FALSE );
         ::EnableWindow( GetDlgItem(IDC_UPDATE), FALSE );   
      }
      else {
         CString s;
         s.Empty();
         m_ctlList.GetItemText(idx,0,s);
         m_ctlMacro.SetWindowText(s);
         s.Empty();
         m_ctlList.GetItemText(idx,1,s);
         m_ctlReplacement.SetWindowText(s);

         ::EnableWindow( GetDlgItem(IDC_DELETE), TRUE );
         ::EnableWindow( GetDlgItem(IDC_UPDATE), TRUE );
      }
      return 0;
   };

protected:
   void PopulateList()
   {
      ATLASSERT(m_mess);
      ATLASSERT(::IsWindow(m_ctlList));
      USES_CONVERSION;
      m_ctlList.DeleteAllItems();
      int cnt = m_mess->m_Macros.size();
      for( int i=0; i<cnt; i++ ) {
         int idx = m_ctlList.InsertItem(i, W2CT(m_mess->m_Macros[i].szMacro));
         ATLASSERT(idx==i);
         m_ctlList.AddItem(idx, 1, W2CT(m_mess->m_Macros[i].szReplacement));
      };
      ::EnableWindow( GetDlgItem(IDC_DELETE), cnt>0 );
      ::EnableWindow( GetDlgItem(IDC_UPDATE), cnt>0 );
   };
};

#endif // !defined(AFX_MACROSDLG_H__5FF6DF87_D992_4D6C_B87F_733856B87CE0__INCLUDED_)
