
#include "stdafx.h"

#include "resource.h"
#include "FileMess.h"
#include "MessHandler.h"
#include "atlwfile.h"
#include "regexp1.h"

LPCWSTR FILEID = L"FileMess";
LPCWSTR APPPATH = L"%APPPATH%";


CMessHandler::CMessHandler()
{
   TCHAR szPath[MAX_PATH];
   ::GetModuleFileName(NULL, szPath, MAX_PATH);
   LPTSTR p = _tcsrchr(szPath, _T('\\'));
   ATLASSERT(p);
   *p = _T('\0');
   m_sAppPath = szPath;

   m_bModified = false;
   LoadState();
   
   m_nThreadCount = 0;
   m_hEvent = ::CreateEvent(NULL, TRUE, FALSE, _T("MessNotifyEvent"));
};

CMessHandler::~CMessHandler()
{
   SaveState();
   Close();
   if( m_hEvent!=NULL ) ::CloseHandle(m_hEvent);
   m_hEvent = NULL;
};


// ----------------------------------------------------


LPCTSTR REG_KEY = _T("SOFTWARE\\BjarkeViksoe\\FileMess");

void CMessHandler::LoadState()
{
   USES_CONVERSION;
   HRESULT Hr;
   DWORD dwVal, dwCount;
   TCHAR szVal[300];

   ::ZeroMemory(&m_options, sizeof(m_options));
   ::ZeroMemory(szVal, sizeof(szVal));

   CRegKey reg;
   reg.Open(HKEY_CURRENT_USER, REG_KEY, KEY_READ);  
   
   Hr = reg.QueryValue(dwVal, _T("RemoveUnmatchedFiles"));
   m_options.bRemoveUnmatchedFiles = Hr==S_OK ? (BOOL)dwVal : FALSE;
   Hr = reg.QueryValue(dwVal, _T("ListenForChanges"));
   m_options.bListenForChanges = Hr==S_OK ? (BOOL)dwVal : TRUE;
   Hr = reg.QueryValue(dwVal, _T("IncludeSubFolders"));
   m_options.bIncludeSubFolders = Hr==S_OK ? (BOOL)dwVal : TRUE;
   Hr = reg.QueryValue(dwVal, _T("IgnoreDuplicates"));
   m_options.bIgnoreDuplicates = Hr==S_OK ? (BOOL)dwVal : TRUE;
   Hr = reg.QueryValue(dwVal, _T("CheckDiskSpace"));
   m_options.bCheckDiskSpace = Hr==S_OK ? (BOOL)dwVal : TRUE;
   Hr = reg.QueryValue(dwVal, _T("UseRecycleBin"));
   m_options.bUseRecycleBin = Hr==S_OK ? (BOOL)dwVal : FALSE;

   *szVal = _T('\0');
   dwCount = sizeof(szVal)/sizeof(TCHAR);
   Hr = reg.QueryValue((LPTSTR)szVal, _T("Extensions"), &dwCount);
   wcscpy( m_options.szExtensions, T2CW(szVal) );

   reg.Close();
};

void CMessHandler::SaveState()
{
   USES_CONVERSION;
   CRegKey reg;
   reg.Create(HKEY_CURRENT_USER, REG_KEY);   
   reg.SetValue(m_options.bRemoveUnmatchedFiles, _T("RemoveUnmatchedFiles"));
   reg.SetValue(m_options.bListenForChanges, _T("ListenForChanges"));
   reg.SetValue(m_options.bIncludeSubFolders, _T("IncludeSubFolders"));
   reg.SetValue(m_options.bIgnoreDuplicates, _T("IgnoreDuplicates"));
   reg.SetValue(m_options.bUseRecycleBin, _T("UseRecycleBin"));
   reg.SetValue(m_options.bCheckDiskSpace, _T("CheckDiskSpace"));
   reg.SetValue(W2CT(m_options.szExtensions), _T("Extensions"));
   reg.Close();
};


// ----------------------------------------------------


void CMessHandler::Init()
{
   USES_CONVERSION;

   Close();

   // Create default macro
   tMacro macro = { 0 };
   wcscpy( macro.szMacro, APPPATH );
   wcscpy( macro.szReplacement, T2CW(m_sAppPath) );
   m_Macros.push_back(macro);

};

void CMessHandler::Close()
{
   RemoveListeners();
   m_strFilePath.Empty();
   m_strFileTitle.Empty();
   m_bModified = false;
   m_Filters.clear();
   m_Macros.clear();
};

bool CMessHandler::Open(LPCTSTR szFilename)
{
   USES_CONVERSION;
   ATLASSERT(szFilename);
   CWaitCursor wait;

   CFile f;
   BOOL bRes = f.Open(szFilename);
   if( !bRes ) return false;

   Init();

   m_strFilePath = szFilename;
   m_strFileTitle = GetFileName(szFilename);

   m_Macros.clear();
   m_Filters.clear();

   DWORD dw;
   long i;

   // Write File ID
   WCHAR szBuffer[64];
   f.Read(szBuffer, wcslen(FILEID)*sizeof(WCHAR));
   szBuffer[wcslen(FILEID)] = L'\0';
   if( wcscmp( szBuffer, FILEID ) != 0 ) {
      CString err;
      err.LoadString(IDS_ERR_INVALIDFILE);
      ::MessageBox(NULL, err, NULL, MB_ICONERROR);
      return false;
   }
   // Write macros
   f.Read(&dw,sizeof(dw));
   for( i=0; i<dw; i++ ) {
      tMacro macro;
      f.Read(&macro, sizeof(macro));
      // Fix the APPPATH macro right away
      if( wcscmp( macro.szMacro, APPPATH ) == 0 ) {
         wcscpy( macro.szReplacement, T2CW(m_sAppPath) );
      }
      m_Macros.push_back(macro);
   };
   // Write mess
   f.Read(&dw,sizeof(dw));
   for( i=0; i<dw; i++ ) {
      tFilter filter = { 0 };
      if( f.Read(&filter, sizeof(filter))==FALSE ) {
         CString err;
         err.LoadString(IDS_ERR_INVALIDSTRUCT);
         ::MessageBox(NULL, err, NULL, MB_ICONERROR);
         return false;
      };
      if( filter.Version > VERSION ) {
         CString err;
         err.LoadString(IDS_ERR_VERSION);
         ::MessageBox(NULL, err, NULL, MB_ICONERROR);
         return false;
      };
      filter.Version = VERSION; // bump to this version
      filter.ID = GetNewID();   // make up a new ID
      filter.nNewFiles = 0;     // reset to be re-calced next time
      filter.nOrigFiles = -1;
      m_Filters.push_back(filter);
   };
   // Done
   f.Close();

   m_bModified = false;

   return true;
};

bool CMessHandler::Save(LPCTSTR szFilename)
{
   ATLASSERT(szFilename);
   USES_CONVERSION;
   CWaitCursor wait;

   m_strFilePath = szFilename;
   if( m_strFilePath.IsEmpty() ) return false;

   long i;
   DWORD dw;

   CFile f;
   BOOL bRes = f.Create(m_strFilePath);
   if( !bRes ) return false;
   
   // Write File ID
   f.Write(FILEID, wcslen(FILEID)*sizeof(WCHAR));
   // Write macros
   dw = m_Macros.size();
   f.Write(&dw,sizeof(dw));
   for( i=0; i<dw; i++ ) {
      f.Write(&m_Macros[i], sizeof(m_Macros[i]));
   };
   // Write mess
   dw = m_Filters.size();
   f.Write(&dw,sizeof(dw));
   for( i=0; i<dw; i++ ) {
      f.Write(&m_Filters[i], sizeof(m_Filters[i]));
   };
   // Done
   f.Close();

   m_bModified = false;

   return true;
};

tFilter CMessHandler::GetDefaultFilter()
{
   SYSTEMTIME time;
   ::GetSystemTime(&time);
   tFilter filter = { 0 };
   filter.Version = VERSION;
   filter.ID = GetNewID();
   filter.action = FILEMOVE;
   filter.Date = filter.DateCreated = filter.DateModified = time;
   filter.nOrigFiles = -1;
   return filter;
};


// ----------------------------------------------------

CString CMessHandler::ParseMacros(LPCTSTR szString)
{
   ATLASSERT(szString);
   CString sString(szString);
   sString.MakeUpper();
   int cnt = m_Macros.size();
   for( int i=0; i<cnt; i++ ) {
      CString sMacro( m_Macros[i].szMacro );
      sMacro.MakeUpper();
      if( sString.Find(sMacro) >= 0 ) {
         int pos = sString.Find(sMacro);
         sString = szString; // BUGFIX: remove the uppercase
         sString = sString.Left(pos) + CString(m_Macros[i].szReplacement) + sString.Mid(pos+sMacro.GetLength());
         return sString;
      };
   }
   return szString;
};

CString CMessHandler::ScanForMacros(LPCTSTR szString)
{
   ATLASSERT(szString);
   CString sString(szString);
   sString.MakeUpper();
   int cnt = m_Macros.size();
   for( int i=0; i<cnt; i++ ) {
      CString sReplacement( m_Macros[i].szReplacement );
      sReplacement.MakeUpper();
      if( sString.Find(sReplacement) >= 0 ) {
         int pos = sString.Find(sReplacement);
         sString = szString; // BUGFIX: remove the uppercase
         sString = sString.Left(pos) + CString(m_Macros[i].szMacro) + sString.Mid(pos+sReplacement.GetLength());
         return sString;
      };
   }
   return szString;
};


// ----------------------------------------------------


typedef struct tagTHREADINFO
{
   HWND hWnd;
   TCHAR szPath[MAX_PATH];
   HANDLE hEvent;
} THREADINFO, *PTHREADINFO;

void __cdecl ThreadFunc(LPVOID pParam)
{
    PTHREADINFO pThreadInfo = (PTHREADINFO)pParam;
    TCHAR szFolder[MAX_PATH];
    _tcscpy( szFolder, pThreadInfo->szPath );
    HANDLE hEvent = pThreadInfo->hEvent;
    HWND hWnd = pThreadInfo->hWnd;
    delete pThreadInfo;
  
    // Get a handle to a file change notification object.
    HANDLE hChange = ::FindFirstChangeNotification((LPCTSTR)szFolder, TRUE, FILE_NOTIFY_CHANGE_FILE_NAME);

    // Return now if ::FindFirstChangeNotification failed.
    if( hChange == INVALID_HANDLE_VALUE ) return;

    HANDLE aHandles[2];
    aHandles[0] = hChange;
    aHandles[1] = hEvent;
    BOOL bContinue = TRUE;

    // Sleep until a file change notification wakes this thread or
    // m_hEvent becomes set indicating it's time for the thread to end.
    while( bContinue ) {
        if( ::WaitForMultipleObjects(2, aHandles, FALSE, INFINITE) - WAIT_OBJECT_0 == 0 ) { 
            // Respond to a change notification.
            ::PostMessage(hWnd, WM_FOLDERCHANGE, 0, 0);
            ::FindNextChangeNotification(hChange);
        }
        else // Kill this thread (m_hEvent became signaled).
            bContinue = FALSE;
    }

    // Close the file change notification handle and return.
    ::FindCloseChangeNotification(hChange);
    
    _endthread();
    return;
}

void CMessHandler::CreateListeners(HWND hWnd)
{
   ATLASSERT(::IsWindow(hWnd));
   ATLASSERT(m_hEvent);
   USES_CONVERSION;

   RemoveListeners();

   if( !m_options.bListenForChanges ) return;

   // Create a listener thread for each of the folder notifications
   // (code ripped from MSDN so no time to create some more fancy stuff).
   // We just make a duplicate folder check to limit the number of threads.
   for( int i=0; i<m_Filters.size(); i++ ) {
      bool bFound = false;
      CString sPath = ParseMacros(W2CT(m_Filters[i].szSourcePath));
      for( int j=0; j<m_FolderNotified.size(); j++ ) {
         if( sPath.CompareNoCase( m_FolderNotified[j] )==0 ) {
            bFound = true;
            break;
         };
      };
      if( !bFound ) {
         CreateMonitoringThread(hWnd, sPath );
      };
   }
};

void CMessHandler::RemoveListeners()
{
   ATLASSERT(m_hEvent);
   // Kill all running file change notification threads.
   if( m_nThreadCount>0 ) {
      ::SetEvent(m_hEvent);
      ::WaitForMultipleObjects(m_nThreadCount, m_hThreads, TRUE, INFINITE);
      for( UINT i=0; i<m_nThreadCount; i++ ) /* */;
      m_nThreadCount = 0;
      m_FolderNotified.clear();
   }
   ::ResetEvent(m_hEvent);
};

void CMessHandler::CreateMonitoringThread(HWND hWnd, LPCTSTR szPath)
{
   ATLASSERT(::IsWindow(hWnd));
   ATLASSERT(szPath);
   // Only so many slots available...
   if( m_nThreadCount >= sizeof(m_hThreads)/sizeof(HANDLE) ) return;

   PTHREADINFO pThreadInfo = new THREADINFO; // Thread will delete
   _tcscpy( pThreadInfo->szPath, szPath);
   pThreadInfo->hEvent = m_hEvent;
   pThreadInfo->hWnd = hWnd;

   HANDLE hThread = (HANDLE)_beginthread(ThreadFunc, 200, pThreadInfo);
   if( hThread==(HANDLE)-1 ) {
      delete pThreadInfo;
      return;
   };
   ::SetThreadPriority(hThread, THREAD_PRIORITY_IDLE);

   m_hThreads[m_nThreadCount] = hThread;
   m_FolderNotified.push_back(szPath);
   m_nThreadCount++;
}


// ----------------------------------------------------


CString CMessHandler::GetFileName(LPCTSTR szFilename)
{
   if( szFilename==NULL ) return _T("");
   LPTSTR p = _tcsrchr( szFilename, _T('\\'));
   if( p==NULL ) return _T("");
   return p+1;
};

void CMessHandler::Process(BOOL bForceRefresh/*=TRUE*/)
{
   USES_CONVERSION;

   for( int i=0; i<m_Filters.size(); i++ ) {
      tFilter &filter = m_Filters[i];
      BOOL bChanged = FALSE;
      ProcessEntry(filter, &bChanged, W2CT(filter.szSourcePath));
      if( bForceRefresh || bChanged ) {
         ProcessStats(filter);
      }
   };

   // Ok, now is the change to delete all "unmatced files"...
   if( m_options.bRemoveUnmatchedFiles ) {
      for( int i=0; i<m_Filters.size(); i++ ) {
         tFilter &filter = m_Filters[i];
         CString sPath;
         sPath.Format(_T("%ls\\*.*"), (LPCTSTR)filter.szSourcePath);
         sPath += _T('\0'); // add that extra NULL for file operation
         SHFILEOPSTRUCT op = { 0 };
         op.hwnd = ::GetActiveWindow();
         op.wFunc = FO_DELETE;
         op.fFlags = FOF_NOCONFIRMATION | FOF_SILENT | FOF_NOERRORUI;
         op.fFlags |= m_options.bUseRecycleBin ? FOF_ALLOWUNDO : 0; 
         op.pFrom = sPath;
         ::SHFileOperation(&op);
      };
   };
};

bool CMessHandler::ProcessEntry(tFilter &filter, BOOL *bChanged, LPCTSTR szSourcePath/*=NULL*/)
{
   USES_CONVERSION;
   int i;

   if( szSourcePath==NULL )
      szSourcePath = W2CT(filter.szSourcePath);

   CString sSourcePath = ParseMacros(szSourcePath);
   CString sTargetPath = ParseMacros(W2CT(filter.szTargetPath));

   // Book-keep files we want to delete and recurse
   vector<CString> vecFilesToDelete;
   vector<CString> vecFolders;

   // Scan the directory
   CFindFile ff;
   BOOL bRes = ff.FindFile( sSourcePath + _T("\\*.*") );
   ATLASSERT(bRes); // it is usually a bug (but may be empty drive)

   for( ; bRes; bRes = ff.FindNextFile() ) {
      if( ff.IsDots() ) continue;
      
      // Don't look at folders now...
      if( ff.IsDirectory() ) {
         if( m_options.bIncludeSubFolders ) {
            vecFolders.push_back(ff.GetFilePath());
         }
         continue;
      }

      // Match pattern
      if( !MatchPatterns(ff.GetFileName(), W2CT(filter.szPattern)) ) continue;

      // Attempt to open the file with some bold access rights.
      // This will ensure that the file is not locked (and not
      // completely written).
      CFile f;
      BOOL bFileReady = f.Open(ff.GetFilePath(), GENERIC_READ | GENERIC_WRITE, 0 /*DENY ALL*/);
      f.Close();
      if( !bFileReady ) continue;

      // Let's just create the directory
      // BUG: This will not create nested folders!!!
      ::CreateDirectory(sTargetPath, NULL);

      // Ok, ready for some file action...
      switch( filter.action ) {
      case FILEMOVE:
         if( CheckDiskSpace(sTargetPath)==false ) continue;
         vecFilesToDelete.push_back( ff.GetFilePath() );
         // FALL THROUGH...
      case FILECOPY:
         {
            if( CheckDiskSpace(sTargetPath)==false ) continue;
            CString sTargetFile;
            sTargetFile = sTargetPath + CString(_T("\\")) + ff.GetFileName();
            BOOL bRes = ::CopyFile(ff.GetFilePath(), sTargetFile, m_options.bIgnoreDuplicates);
            ATLASSERT(bRes);
         };
         break;
      case FILEDELETE:
         {
            vecFilesToDelete.push_back( ff.GetFilePath() );
         };
         break;
      default:
         ATLASSERT("Invalid ACTION"==NULL);
      };
      *bChanged = TRUE;
   }
   ff.Close();


   // Now remove the marked files
   for( i=0; i<vecFilesToDelete.size(); i++ ) {
      CString sFile = vecFilesToDelete[i];
      if( m_options.bUseRecycleBin ) {
         CString sNewFile = sFile + _T('\0'); // add that extra NULL for file operation
         SHFILEOPSTRUCT op = { 0 };
         op.hwnd = ::GetActiveWindow();
         op.wFunc = FO_DELETE;
         op.fFlags = FOF_ALLOWUNDO | FOF_NOCONFIRMATION | FOF_SILENT | FOF_NOERRORUI;
         op.pFrom = sNewFile;
         int res = ::SHFileOperation(&op);
         ATLASSERT(res==0);
      }
      else {
         // Do standard delete (this will minimize the
         // errors and notifications!)
         BOOL res = ::DeleteFile(sFile);
         ATLASSERT(res);
      }
   };

   // Notify shell of changes
   if( *bChanged ) {
      // Behave correctly and notify shell
      ::SHChangeNotify(SHCNE_UPDATEDIR, SHCNF_PATH, sTargetPath, 0);
      if( vecFilesToDelete.size() > 0 ) {
         // NOPE: This will trigger a new refresh!
         //::SHChangeNotify(SHCNE_UPDATEDIR, SHCNF_PATH, sSourcePath, 0);
      };
   }

   // Recurse sub-folders
   for( i=0; i<vecFolders.size(); i++ ) {
      ProcessEntry(filter, bChanged, vecFolders[i]);
   };

   return true;
};

bool CMessHandler::CheckDiskSpace(LPCTSTR szPath)
// Check disk space. This is always nasty because of
// the missing GetDiskFreeSpaceEx() function.
{
#define LOWSPACE 1024*1024*5
   ATLASSERT(szPath);

   if( !m_options.bCheckDiskSpace ) return true;

   typedef BOOL (WINAPI *GETDISKFREEPROC)(LPCTSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER);
#ifndef _UNICODE
   GETDISKFREEPROC pGetDiskFreeSpaceEx = (GETDISKFREEPROC)::GetProcAddress( ::GetModuleHandle("kernel32.dll"),
                            "GetDiskFreeSpaceExA");
#else
   GETDISKFREEPROC pGetDiskFreeSpaceEx = (GETDISKFREEPROC )::GetProcAddress( ::GetModuleHandle("kernel32.dll"),
                            "GetDiskFreeSpaceExW");
#endif
   BOOL fResult;
   bool bIsLow = false;
   if( pGetDiskFreeSpaceEx!=NULL ) {
      ULARGE_INTEGER i64FreeBytesToCaller;
      ULARGE_INTEGER i64TotalBytes;
      ULARGE_INTEGER i64FreeBytes;
      fResult = pGetDiskFreeSpaceEx(szPath,
                   (PULARGE_INTEGER)&i64FreeBytesToCaller,
                   (PULARGE_INTEGER)&i64TotalBytes,
                   (PULARGE_INTEGER)&i64FreeBytes);
      // Process GetDiskFreeSpaceEx results.
      if( fResult ) 
         bIsLow = (i64FreeBytesToCaller.LowPart < LOWSPACE) && (i64FreeBytesToCaller.HighPart==0);
   }
   else {
      DWORD dwSectPerClust;
      DWORD dwBytesPerSect;
      DWORD dwFreeClusters;
      DWORD dwTotalClusters;
      TCHAR szDrive[3] = { _T("?:") };
      szDrive[0] = szPath[0];
      fResult = ::GetDiskFreeSpace(szDrive, 
                   &dwSectPerClust, 
                   &dwBytesPerSect,
                   &dwFreeClusters, 
                   &dwTotalClusters);
      // Process GetDiskFreeSpace results.
      if( fResult ) 
         bIsLow = dwFreeClusters*dwSectPerClust*dwBytesPerSect < LOWSPACE;
   }
   if( !bIsLow ) return true;

   // In case of a low disk space situatio, we halt the
   // system with a modal message box - urging the user
   // to free some space.

   // Check if we have warned once before
   static bool bDisplayed = false;
   if( bDisplayed ) return false; // don't warn again, but skip copy anyway
   bDisplayed = true;

   // Bring up message
   CString err;
   err.LoadString(IDS_ERR_LOWDISKSPACE);
   ::MessageBox(NULL, err, NULL, MB_ICONERROR|MB_SYSTEMMODAL);
   return false;
};

bool CMessHandler::ProcessStats(tFilter &filter)
{
   USES_CONVERSION;

   CString sTargetPath = ParseMacros(W2CT(filter.szTargetPath));

   CFindFile ff;
   BOOL bRes = ff.FindFile( sTargetPath + _T("\\*.*") );
   // Prepare filter for scan
   filter.nFiles = 0;
   filter.dwTotalFileSize = 0;
   for( ; bRes; bRes = ff.FindNextFile() ) {
      if( ff.IsDots() ) continue;
      if( ff.IsDirectory() ) continue;
      if( !MatchPatterns(ff.GetFileName(), W2CT(filter.szPattern)) ) continue;
      filter.nFiles++;
      filter.dwTotalFileSize += ff.GetFileSize();
   }
   ff.Close();

   // Update files count statistics
   if( filter.nOrigFiles < 0 ) filter.nOrigFiles = filter.nFiles;
   filter.nNewFiles = filter.nFiles - filter.nOrigFiles;
   if( filter.nNewFiles < 0 ) {
      filter.nNewFiles = 0;
   };

   return true;
};

