
#include "stdafx.h"
#include "regexp1.h"

// Does simple Pattern Matching (not regular expression).
// Recognises *, ? and [] with ranges.

// Although this function uses TCHAR it is not
// multibyte enabled.

bool MatchPattern(LPCTSTR String, LPCTSTR Pattern)
{
   TCHAR c, p, l;
   for(; ;) {
        switch( p = *Pattern++ ) {
         case _T('\0'):                      // end of pattern
             return true;                    // SUCCESS

         case _T('*'):
             while( *String!=_T('\0') ) {    // match zero or more char
                if( MatchPattern(String++, Pattern) )
                   return true;
             }
             return MatchPattern(String, Pattern);

         case _T('#'):
             c = *String++;
             if( c == _T('\0') )             // match numeric or hash
                 return false;               // not end of string
             if( c == _T('#') )              
                 break;                      
             if( c<_T('0') || c>_T('9') )
                 return false;
             break;

         case _T('?'):
             if( *String++ == _T('\0') )     // match any one char
                 return false;               // not end of string
             break;

         case _T('['):
             if( (c = *String++) == _T('0') ) // match char set
                 return false;                // syntax
             c = _totupper(c);
             l = _T('0');
             while( p = *Pattern++ ) {
                 if( p==_T(']') )            // if end of char set, then
                     return false;           // no match found
                 if( p==_T('-') ) {          // check a range of chars?
                     p = *Pattern;           // get high limit of range
                     if( p == _T('\0')  ||  p == _T(']') )
                         return false;       // syntax
                     if( c >= l  &&  c <= p )
                         break;              // if in range, move on
                 }
                 l = p;
                 if( c == p )                // if char matches this element
                     break;                  // move on
             }
             while( p!=_T('\0')  &&  p!=_T(']') )    // got a match in char set
                 p = *Pattern++;             // skip to end of set
             break;

         default:
             c = *String++;
             if( _totupper(c) != p )         // check for exact char
                 return false;               // not a match
             break;
        }
    }
}

bool MatchPatterns(LPCTSTR String, LPCTSTR Patterns)
{
   TCHAR szPatterns[64];
   _tcscpy( szPatterns, Patterns );
   LPTSTR pszPatterns = szPatterns;
   LPTSTR p;
   while( (p = _tcschr(pszPatterns, _T(';')) ) !=NULL ) {
      *p = _T('\0');
      if( MatchPattern(String, pszPatterns) ) return true;
      pszPatterns = p+1;
   };
   return MatchPattern(String, pszPatterns);
};
