// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__1593B7A1_AB63_499E_ADB7_3CF8F5BAF5BD__INCLUDED_)
#define AFX_MAINFRM_H__1593B7A1_AB63_499E_ADB7_3CF8F5BAF5BD__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "FileMess.h"
#include "MessHandler.h"
#include "Extensions.h"

#define ID_CMD_ADDIN 35000
#define FILE_EXTENSION _T("mess")

LPCTSTR lpcstrFilter = 
   _T("FileMess Files (*.mess)\0*.mess\0")
   _T("All Files (*.*)\0*.*\0")
   _T("");


class CMainFrame : 
   public CFrameWindowImpl<CMainFrame>, 
   public CUpdateUI<CMainFrame>,
   public CMessageFilter, 
   public CIdleHandler
{
public:
   DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

   CFileMessView m_view;
   CString m_strAppPath;
   CMessHandler m_mess;
   vector<CExtension> m_extensions;
   CLIPFORMAT m_CF;
   CCommandBarCtrl m_CmdBar;
   int iSortIndex;
   bool bSortAsc;

   virtual BOOL PreTranslateMessage(MSG* pMsg)
   {
      if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg))
         return TRUE;
      return m_view.PreTranslateMessage(pMsg);
   }

   virtual BOOL OnIdle()
   {
      UIEnable(ID_FILE_SAVE, m_mess.IsModified());
      int nFilters = m_mess.m_Filters.size();
      UIEnable(ID_FILE_SAVE_AS, nFilters>0);
      UIEnable(ID_TOOLS_REFRESH, nFilters>0);

      tFilter *filter = GetSelectedFilter();
      UIEnable(ID_EDIT_PROPERTIES, filter!=NULL);
      UIEnable(ID_EDIT_BROWSE, filter!=NULL);
      UIEnable(ID_EDIT_COPY, filter!=NULL);
      UIEnable(ID_EDIT_DELETE, filter!=NULL);

      UIEnable(ID_EDIT_PASTE, ::IsClipboardFormatAvailable(m_CF));

      UIUpdateToolBar();
      return FALSE;
   }

   BEGIN_MSG_MAP(CMainFrame)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_CLOSE, OnClose)
      COMMAND_ID_HANDLER(ID_APP_EXIT, OnFileExit)
      COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
      COMMAND_ID_HANDLER(ID_FILE_OPEN, OnFileOpen)
      COMMAND_ID_HANDLER(ID_FILE_SAVE, OnFileSave)
      COMMAND_ID_HANDLER(ID_FILE_SAVE_AS, OnFileSaveAs)
      COMMAND_ID_HANDLER(ID_EDIT_ADD, OnFilterAdd)
      COMMAND_ID_HANDLER(ID_EDIT_PROPERTIES, OnFilterProperties)
      COMMAND_ID_HANDLER(ID_EDIT_BROWSE, OnFilterBrowse)
      COMMAND_ID_HANDLER(ID_EDIT_DELETE, OnFilterDelete)
      COMMAND_ID_HANDLER(ID_EDIT_COPY, OnEditCopy)
      COMMAND_ID_HANDLER(ID_EDIT_PASTE, OnEditPaste)
      COMMAND_ID_HANDLER(ID_VIEW_TOOLBAR, OnViewToolBar)
      COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
      COMMAND_ID_HANDLER(ID_VIEW_OPTIONS, OnViewOptions)
      COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
      COMMAND_ID_HANDLER(ID_TOOLS_REFRESH, OnToolsRefresh)
      COMMAND_ID_HANDLER(ID_TOOLS_MACROS, OnToolsMacros)
      MESSAGE_HANDLER(WM_FOLDERCHANGE, OnFolderChange)
      MESSAGE_HANDLER(WM_REFRESH, OnRefresh)
      COMMAND_RANGE_HANDLER(ID_CMD_ADDIN, ID_CMD_ADDIN+50, OnToolsExecute);
      CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
      CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
      CHAIN_COMMANDS_ALT_MEMBER((m_view), 1)
      NOTIFY_CODE_HANDLER(LVN_ITEMCHANGED, OnListSelect)
      NOTIFY_CODE_HANDLER(NM_DBLCLK, OnListDblClick)
      NOTIFY_CODE_HANDLER(LVN_COLUMNCLICK, OnColumnClick)
      MESSAGE_HANDLER(WM_CONTEXTMENU, OnContextMenu)
   END_MSG_MAP()

   BEGIN_UPDATE_UI_MAP(CMainFrame)
      UPDATE_ELEMENT(ID_VIEW_TOOLBAR, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_VIEW_STATUS_BAR, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_FILE_SAVE, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_FILE_SAVE_AS, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_EDIT_PROPERTIES, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_EDIT_BROWSE, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_EDIT_DELETE, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_EDIT_COPY, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_EDIT_PASTE, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
      UPDATE_ELEMENT(ID_TOOLS_REFRESH, UPDUI_MENUPOPUP | UPDUI_TOOLBAR)
   END_UPDATE_UI_MAP()

   LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      TCHAR szPath[MAX_PATH];
      ::GetModuleFileName(NULL, szPath, MAX_PATH);
      LPTSTR p = _tcsrchr(szPath, _T('\\'));
      ATLASSERT(p);
      *p = _T('\0');
      m_strAppPath = szPath;

      // Get Clipboard format id for app.
      UINT m_CF = ::RegisterClipboardFormat(_T("FileMess"));

      // Create command bar window
      HWND hWndCmdBar = m_CmdBar.Create(m_hWnd, rcDefault, NULL, ATL_SIMPLE_CMDBAR_PANE_STYLE);
      // Add extension menus
      AttachExtensionMenus(GetMenu());
      // Attach menu
      m_CmdBar.AttachMenu(GetMenu());
      // Load command bar images
      m_CmdBar.LoadImages(IDR_MAINFRAME);
      // Remove old menu
      SetMenu(NULL);

      HWND hWndToolBar = CreateSimpleToolBarCtrl(m_hWnd, IDR_MAINFRAME, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);
      ATLASSERT(::IsWindow(hWndToolBar));
      CreateSimpleReBar(ATL_SIMPLE_REBAR_NOBORDER_STYLE);
      AddSimpleReBarBand(hWndCmdBar);
      AddSimpleReBarBand(hWndToolBar, NULL, TRUE);
      CreateSimpleStatusBar();

      DWORD dwStyle = 
         WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | LVS_REPORT | LVS_SHOWSELALWAYS | LVS_SINGLESEL;
      DWORD dwExStyle = 
         WS_EX_CLIENTEDGE;
      m_hWndClient = m_view.Create(m_hWnd, rcDefault, NULL, dwStyle, dwExStyle);
      ATLASSERT(::IsWindow(m_hWndClient));
      iSortIndex = 0;
      bSortAsc = true;
      m_view.SortItems(ListViewCompareProc, (DWORD)this);

      // We want a different size
      int nWidth = 200+100+80+80;
      CRect rc;
      GetWindowRect(&rc);
      rc.right = rc.left + nWidth + 30;
      MoveWindow(&rc, FALSE);

      UIAddToolBar(hWndToolBar);
      UISetCheck(ID_VIEW_TOOLBAR, 1);
      UISetCheck(ID_VIEW_STATUS_BAR, 1);

      CMessageLoop* pLoop = _Module.GetMessageLoop();
      pLoop->AddMessageFilter(this);
      pLoop->AddIdleHandler(this);

      BOOL bDummy;
      OnFileNew(0,0,NULL,bDummy);

      return 0;
   }

   LRESULT OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
   {
      if( m_mess.IsModified() ) {
         CString s;
         s.LoadString(IDS_SAVEFILE);
         CString sTitle;
         sTitle.LoadString(IDR_MAINFRAME);
         if( MessageBox(s,sTitle,MB_YESNO | MB_ICONQUESTION) == IDYES ) {
            BOOL bDummy;
            OnFileSave(0,0,NULL,bDummy);
         }
      };
      bHandled = FALSE;
      return 0;
   };

   LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      PostMessage(WM_CLOSE);
      return 0;
   }

   LRESULT OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      if( m_mess.IsModified() ) {
         CString s;
         s.LoadString(IDS_SAVEFILE);
         CString sTitle;
         sTitle.LoadString(IDR_MAINFRAME);
         if( MessageBox(s,sTitle,MB_YESNO | MB_ICONQUESTION) == IDYES ) {
            BOOL bDummy;
            OnFileSave(0,0,NULL,bDummy);
         }
      };
      Init();
      return 0;
   }

   LRESULT OnFileOpen(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      BOOL bDummy;
      OnFileNew(0,0,NULL,bDummy);
      
      CFileDialog dlg(TRUE, FILE_EXTENSION, m_strAppPath + _T("\\Files.mess"), OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, lpcstrFilter);
      int nRet = dlg.DoModal();
      if(nRet == IDOK) {
         Open(dlg.m_ofn.lpstrFile);
      }

      return 0;
   }

   LRESULT OnFileSave(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      if( !m_mess.IsModified() ) return 0;
      BOOL bDummy;
      if( m_mess.FileTitle().IsEmpty() ) 
         OnFileSaveAs(0,0,NULL,bDummy);
      else
         Save(m_mess.FileName());
      return 0;
   }

   LRESULT OnFileSaveAs(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CFileDialog dlg(FALSE, FILE_EXTENSION, m_mess.FileName(), OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, lpcstrFilter);
      int nRet = dlg.DoModal();
      if(nRet == IDOK) {
         BOOL bRet = Save(dlg.m_ofn.lpstrFile);
      }

      return 0;
   }

   LRESULT OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      static BOOL bNew = TRUE;   // initially visible
      bNew = !bNew;
      ::SendMessage(m_hWndToolBar, RB_SHOWBAND, 1, bNew);   // toolbar is band #1
      UISetCheck(ID_VIEW_TOOLBAR, bNew);
      UpdateLayout();
      return 0;
   }

   LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      BOOL bNew = !::IsWindowVisible(m_hWndStatusBar);
      ::ShowWindow(m_hWndStatusBar, bNew ? SW_SHOWNOACTIVATE : SW_HIDE);
      UISetCheck(ID_VIEW_STATUS_BAR, bNew);
      UpdateLayout();
      return 0;
   }

   LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CAboutDlg dlg;
      dlg.DoModal();
      return 0;
   }

   LRESULT OnViewOptions(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& bHandled)
   {
      COptionsDlg dlg;
      dlg.m_options = &m_mess.m_options;
      dlg.DoModal();
      // Commit changes
      m_mess.CreateListeners(m_hWnd); // may have changes listener mode
      PostMessage(WM_REFRESH, FALSE, 0);
      return 0;
   }

   LRESULT OnToolsMacros(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CMacrosDlg dlg;
      dlg.m_mess = &m_mess;
      dlg.DoModal();
      return 0;
   }

   LRESULT OnToolsRefresh(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      PostMessage(WM_REFRESH, TRUE, 0);
      return 0;
   }

   LRESULT OnToolsExecute(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& bHandled)
   {
      CWaitCursor wait;
      CString s;
      // Update statusbar
      s.LoadString(IDS_PROCESSING);
      ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (255), (LPARAM)(LPCTSTR)s);
      // Look for the add-in
      for( int i=0; i<m_extensions.size(); i++ ) {
         if( m_extensions[i].GetID()==wID ) {
            // Prepare add-in run
            tFilter *filter = GetSelectedFilter();
            UINT SelectedID = ( filter==NULL ? -1 : filter->ID );
            m_extensions[i].Execute(m_hWnd, m_mess, SelectedID);
         };
      };
      // Update statusbar
      s.LoadString(ATL_IDS_IDLEMESSAGE);
      ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (255), (LPARAM)(LPCTSTR)s);
      // Reprocess list in case add-in changes something
      PostMessage(WM_REFRESH, TRUE, 0);
      return 0;
   }

//---------------------------------------------

   LRESULT OnFilterAdd(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      // Create new (default) entry
      tFilter filter = m_mess.GetDefaultFilter();

      CEntryDlg dlg;
      dlg.m_filter = &filter;
      dlg.m_mess = &m_mess;
      UINT res = dlg.DoModal();
      if( res==IDOK ) {
         m_mess.m_Filters.push_back(filter);
         m_mess.Process(FALSE);
         m_mess.SetModified();
         m_mess.CreateListeners(m_hWnd);
         UpdateList();
         SelectFilter(filter);
      };

      return 0;
   }

   LRESULT OnFilterDelete(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      tFilter *filter = GetSelectedFilter();
      if( filter==NULL ) return 0;

      for( int i=0; i<m_mess.m_Filters.size(); i++ ) {
         if( filter->ID == m_mess.m_Filters[i].ID ) {
            m_mess.m_Filters.erase(m_mess.m_Filters.begin()+i);
         }
      };

      m_mess.SetModified();
      m_mess.CreateListeners(m_hWnd);
      UpdateList();

      return 0;
   }

   LRESULT OnFilterBrowse(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      tFilter *filter = GetSelectedFilter();
      if( filter==NULL ) return 0;

      USES_CONVERSION;
      CString sPath = m_mess.ParseMacros(W2CT(filter->szTargetPath));
      ::ShellExecute(m_hWnd, _T("open"), sPath, NULL, sPath, SW_SHOWNORMAL);
      return 0;
   }

   LRESULT OnFilterProperties(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      SYSTEMTIME time;
      ::GetSystemTime(&time);

      // Find selected entry...
      tFilter *filter = GetSelectedFilter();
      if( filter==NULL ) return 0;

      CEntryDlg dlg;
      dlg.m_filter = filter;
      dlg.m_mess = &m_mess;
      UINT res = dlg.DoModal();
      if( res==IDOK ) {
         CWaitCursor wait;
         filter->DateModified = time;
         m_mess.Process(FALSE);
         m_mess.SetModified();
         m_mess.CreateListeners(m_hWnd);
         UpdateList();
         SelectFilter(*filter);
      };

      return 0;
   }

   LRESULT OnFolderChange(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
   {
      // Quick hack to remove all messages awaiting
      MSG msg;
      while( ::PeekMessage(&msg, NULL, WM_FOLDERCHANGE, WM_FOLDERCHANGE, PM_REMOVE)) ;

      PostMessage(WM_REFRESH, FALSE, 0);
      return TRUE;
   };

   LRESULT OnRefresh(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      CWaitCursor wait;
      CString s;
      // Update statusbar
      s.LoadString(IDS_PROCESSING);
      ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (255), (LPARAM)(LPCTSTR)s);
      // Do it...
      m_mess.Process((BOOL)wParam);
      int idx = m_view.GetSelectedIndex();
      UpdateList();
      m_view.SelectItem(idx);
      m_view.EnsureVisible(idx,FALSE);
      for( int i=0; i<m_extensions.size(); i++ ) m_extensions[i].Refresh(m_hWnd, m_mess);
      // Update statusbar
      s.LoadString(ATL_IDS_IDLEMESSAGE);
      ::SendMessage(m_hWndStatusBar, SB_SETTEXT, (255), (LPARAM)(LPCTSTR)s);
      return 0;
   }


//---------------------------------------------


   LRESULT OnEditCopy(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      USES_CONVERSION;
      tFilter *pFilter = GetSelectedFilter();
      if( pFilter==NULL ) return 0;

      // Open the clipboard...
      if( !::OpenClipboard(m_hWnd) ) return 0;
      // Empty anything already there...
      EmptyClipboard();
      // Allocate global memory for transfer...
      HGLOBAL hData = ::GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, sizeof(*pFilter));
      if( hData==NULL ) {
         ::CloseClipboard();
         return 0;
      };
      // Put our string in the global memory...
      LPBYTE ptr = (LPBYTE)::GlobalLock(hData);
      ::CopyMemory(ptr, pFilter, sizeof(*pFilter));
      ::GlobalUnlock(hData);
      // Put data on the clipboard!
      ::SetClipboardData(m_CF, hData);
      ::CloseClipboard();
      
      return 0;
   };

   LRESULT OnEditPaste(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      CWaitCursor wait;
      tFilter filter;

      // Open the clipboard...
      ::OpenClipboard(m_hWnd);
      // Get the data
      HGLOBAL hData = ::GetClipboardData(m_CF);
      if( hData==NULL ) {
         ::CloseClipboard();
        return 0;
      }
      LPBYTE ptr = (LPBYTE)::GlobalLock(hData);
      if( ptr==NULL ) {
         // Free memory...
         ::GlobalFree(hData);
         ::CloseClipboard();
         return 0;
      };
      ::CopyMemory(&filter, ptr, sizeof(filter));
      ::GlobalUnlock(hData);
      ::CloseClipboard();

      // Add the copy to the list of filters
      // Make sure to update the system
      SYSTEMTIME time;
      ::GetSystemTime(&time);
      filter.DateCreated = filter.DateModified = time;
      filter.ID = GetNewID();
      m_mess.m_Filters.push_back(filter);
      m_mess.Process(FALSE);
      m_mess.SetModified();
      m_mess.CreateListeners(m_hWnd);
      UpdateList();
      SelectFilter(filter);

      return 0;
   };


//---------------------------------------------


   LRESULT OnColumnClick(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
   {
      LPNMLISTVIEW pnmv = (LPNMLISTVIEW)pnmh;
      if( pnmv->iSubItem==iSortIndex ) bSortAsc = !bSortAsc;
      iSortIndex = pnmv->iSubItem;
      m_view.SortItems(ListViewCompareProc, (DWORD)this);
      bHandled = FALSE;
      return 0;
   };

   LRESULT OnListSelect(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
   {
      return 0;
   };

   LRESULT OnListDblClick(int idCtrl, LPNMHDR pnmh, BOOL& bHandled)
   {
      return OnFilterProperties(0,0,NULL, bHandled);
   };

   LRESULT OnContextMenu(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
      OnIdle(); // Bug in WTL prevent correct UI update. Force it.
      if( GetSelectedFilter()==NULL )
         bHandled = FALSE;
      else if((HWND)wParam == m_view.m_hWnd) {
         CMenu menuContext;
         menuContext.LoadMenu(IDR_POPUP);
         CMenuHandle menuPopup(menuContext.GetSubMenu(0));
         m_CmdBar.TrackPopupMenu(menuPopup, TPM_LEFTALIGN | TPM_RIGHTBUTTON, LOWORD(lParam), HIWORD(lParam));
      }
      else
         bHandled = FALSE;
      return 0;
   }


//---------------------------------------------

   void AttachExtensionMenus(HMENU hMenu)
   {
      CMenuHandle menu(hMenu);
      CMenuHandle toolmenu( menu.GetSubMenu(3) );

      // For all files in the application path with the extension
      // ".ext", load them (as DLLs) and ask for their caption, then
      // add them to the menu...
      CFindFile ff;
      BOOL bOk = ff.FindFile(m_strAppPath + _T("\\*.ext"));
      for( int i=0; bOk; bOk = ff.FindNextFile(), i++ ) {
         if( ff.IsDots() ) continue;
         if( ff.IsDirectory() ) continue;
         CExtension ext;
         if( ext.Load(ff.GetFilePath())==TRUE ) {
            CString sTitle = ext.GetTitle();
            if( sTitle.IsEmpty() ) continue; // hmm?
            ext.SetID( ID_CMD_ADDIN+i );
            m_extensions.push_back(ext);
            toolmenu.InsertMenu( -1, 0, ext.GetID(), sTitle );
         }
      };
   };

   void Init()
   {
      m_mess.Init();
      m_mess.RemoveListeners();
      UpdateList();
      for( int i=0; i<m_extensions.size(); i++ ) m_extensions[i].ProjectInit(m_hWnd, m_mess);
   };

   bool Open(LPCTSTR szFilename)
   {
      if( m_mess.Open(szFilename)==false ) return false;
      CWaitCursor wait;
      ::SHAddToRecentDocs(SHARD_PATH, szFilename);
      m_mess.Process(TRUE);
      m_mess.CreateListeners(m_hWnd);
      UpdateList();
      for( int i=0; i<m_extensions.size(); i++ ) m_extensions[i].ProjectInit(m_hWnd, m_mess);
      return true;
   };

   bool Save(LPCTSTR szFilename)
   {
      CWaitCursor wait;
      if( m_mess.Save(szFilename)==false ) return false;
      // Look for the add-in
      for( int i=0; i<m_extensions.size(); i++ ) m_extensions[i].Save(m_hWnd, m_mess);
      return true;

   };

   void UpdateList()
   {
      ATLASSERT(::IsWindow(m_view));
      USES_CONVERSION;
      // Recreate the list view contents...
      m_view.SetRedraw(FALSE);
      m_view.DeleteAllItems();
      for( int i=0; i<m_mess.m_Filters.size(); i++ ) {
         tFilter &filter = m_mess.m_Filters[i];
         CString s;
         int idx = m_view.InsertItem(i, W2CT(filter.szTitle) );
         ATLASSERT(i==idx);
         m_view.SetItemData(idx, filter.ID );
         m_view.AddItem(idx, 1, W2CT(filter.szPattern) );
         s.Format(_T("%d"), filter.nFiles);
         m_view.AddItem(idx, 2, (LPCTSTR)s );
         s.Format(_T("%d"), filter.nNewFiles);
         m_view.AddItem(idx, 3, (LPCTSTR)s );
      };
      m_view.SortItems(ListViewCompareProc, (DWORD)this);
      m_view.SetRedraw(TRUE);
   };

   tFilter *GetSelectedFilter()
   {
      ATLASSERT(::IsWindow(m_view));
      // Find selected listview filter...
      LVITEM itm = { 0 };
      itm.mask = LVIF_PARAM;
      if( m_view.GetSelectedItem(&itm)==FALSE ) return NULL;
      for( int i=0; i<m_mess.m_Filters.size(); i++ ) {
         if( itm.lParam == m_mess.m_Filters[i].ID ) {
            return &(m_mess.m_Filters[i]);
         }
      };
      return NULL;
   };

   void SelectFilter(tFilter &filter)
   {
      // Select a specific filter in the listview
      ATLASSERT(::IsWindow(m_view));
      int cnt = m_view.GetItemCount();
      for( int i=0; i<cnt; i++ ) {
         if( m_view.GetItemData(i)==filter.ID ) {
            m_view.SelectItem(i);
            break;
         };
      };
   };

};

int CALLBACK ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
   CMainFrame *pThis = (CMainFrame *)lParamSort;
   ATLASSERT(pThis);

   tFilter *pFilters = &(pThis->m_mess.m_Filters[0]);
   
   int idx1 = -1;
   int idx2 = -1;
   for( int i=0; i<pThis->m_mess.m_Filters.size(); i++ ) {
      if( (DWORD)lParam1==pFilters[i].ID ) idx1 = i;
      if( (DWORD)lParam2==pFilters[i].ID ) idx2 = i;
   };
   if( idx1<0 || idx2<0 ) return 0;

   int cmp = 0;
   switch( pThis->iSortIndex ) {
   case 0:
      cmp = wcscmp(pFilters[idx1].szTitle, pFilters[idx2].szTitle );
      break;
   case 1:
      cmp = wcscmp(pFilters[idx1].szPattern, pFilters[idx2].szPattern );
      break;
   case 2:
      cmp = pFilters[idx1].nFiles - pFilters[idx2].nFiles;
      break;
   case 3:
      cmp = pFilters[idx1].nNewFiles - pFilters[idx2].nNewFiles;
      break;
   default:
      ATLASSERT("Invalid column index"==NULL);
   };
   if( !pThis->bSortAsc ) cmp = -cmp;

   return cmp;
}


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__1593B7A1_AB63_499E_ADB7_3CF8F5BAF5BD__INCLUDED_)
